import express from 'express'
import session from 'express-session'
import bodyParser from 'body-parser'
import exphbs from 'express-handlebars'
// import controllers from './controllers'
import path from 'path'
import busboy from 'connect-busboy'
import portfinder from 'portfinder'
import clc from 'cli-color'
import openurl from 'openurl'

import {
  config,
  FileParser,
  fileUtils,
  printInput,
  abeImport,
  testObj,
  math,
  notEmpty,
  printJson,
  className,
  abe,
  compileAbe,
  listPage,
  ifIn,
  ifCond,
  moduloIf,
  attrAbe,
  folders,
  Plugins,
  printConfig,
  Hooks
} from '../cli'

var abePort = null

if(process.env.ROOT) config.set({root: process.env.ROOT.replace(/\/$/, '') + '/'})
if(process.env.PORT) abePort = process.env.PORT
config.set({webport: process.env.WEBPORT ? process.env.WEBPORT : 8081})

var html = exphbs.create({
  extname: '.' + config.files.templates.extension,
  helpers: {
    abe: compileAbe,
    listPage: listPage,
    math: math,
    printInput: printInput,
    abeImport: abeImport,
    moduloIf: moduloIf,
    testObj: testObj,
    notEmpty: notEmpty,
    printJson: printJson,
    className: className,
    attrAbe: attrAbe,
    folders: folders,
    printConfig: printConfig,
    ifIn: ifIn,
    ifCond: ifCond
  }
})

portfinder.getPort(function (err, freePort) {
  var app = express()
  var port = (abePort !== null) ? abePort : (freePort || 3000)
  port = Hooks.instance.trigger('beforeExpress', port)

  app.set('views', path.join(__dirname, '/templates'))

  // app.set('template', )
  app.engine('.html', html.engine)
  app.set('view engine', '.html')

  app.locals.layout = false;

  app.use(express.static(__dirname + '/public'))
  FileParser.copySiteAssets()

  var sites = FileParser.getFolders(config.root.replace(/\/$/, ""), false, 0)
  let publish = fileUtils.concatPath(config.root, config.publish.url)
  app.use(express.static(publish))

  if(config.partials !== '') {
    if (fileUtils.isFile(fileUtils.concatPath(config.root, config.partials))) {
      app.use(express.static(fileUtils.concatPath(config.root, config.partials)))
    }
  }

  var pluginsPartials = Plugins.instance.getPartials()
  Array.prototype.forEach.call(pluginsPartials, (pluginPartials) => {
    app.use(express.static(pluginPartials))
  })

  app.use(express.static(__dirname + '/node_modules/handlebars/dist'))
  app.use(busboy())
  app.use(bodyParser.json())
  app.use(bodyParser.urlencoded({extended: true}))
  app.use(session({secret: config.sessionSecret, resave: false, saveUninitialized: true}));
  Hooks.instance.trigger('afterExpress', app, express)

  // important : require here so config.root is defined
  var controllers = require('./controllers')
  app.use(controllers.default)

  app.listen(port, function() {
    console.log(clc.green(`\nserver running at http://localhost:${port}/`))
    if(process.env.OPENURL) openurl.open(`http://localhost:${port}/abe/`)
  })
})
